@@ -46,7 +46,7 @@ namespace NextPVR
       }
       else
       {
-        m_sidUpdate = time(nullptr);
+        RenewSID();
       }
     }
     kodi::Log(ADDON_LOG_DEBUG, "DoRequest return %s %d %d %d", resource.c_str(), resultCode, response.length(), std::chrono::duration_cast<std::chrono::milliseconds>(std::chrono::steady_clock::now() - start));
@@ -66,7 +66,6 @@ namespace NextPVR
     // return is same on timeout or http return ie 404, 500.
     tinyxml2::XMLError retError = tinyxml2::XML_ERROR_FILE_NOT_FOUND;
     std::unique_lock<std::mutex> lock(m_mutexRequest);
-    m_start = time(nullptr);
     // build request string, adding SID if requred
     std::string URL;
 
@@ -121,11 +120,12 @@ namespace NextPVR
         }
         else
         {
-          m_sidUpdate = time(nullptr);
+          RenewSID();
         }
       }
     }
-    kodi::Log(ADDON_LOG_DEBUG, "DoMethodRequest %s %d %d %d", resource.c_str(), retError, response.length(), std::chrono::duration_cast<std::chrono::milliseconds>(std::chrono::steady_clock::now() - start));
+    int milliseconds = static_cast<int>(std::chrono::duration_cast<std::chrono::milliseconds>(std::chrono::steady_clock::now() - start).count());
+    kodi::Log(ADDON_LOG_DEBUG, "DoMethodRequest %s %d %d %d", resource.c_str(), retError, response.length(), milliseconds);
     return retError;
   }
 
