@@ -82,7 +82,20 @@ PVR_ERROR Timers::GetTimers(kodi::addon::PVRTimersResultSet& results)
       tinyxml2::XMLNode* pRulesNode = pMatchRulesNode->FirstChildElement("Rules");
 
       tag.SetClientIndex(XMLUtils::GetUIntValue(pRecurringNode, "id"));
-      tag.SetClientChannelUid(XMLUtils::GetIntValue(pRulesNode, "ChannelOID"));
+      int channelUID = XMLUtils::GetIntValue(pRulesNode, "ChannelOID");
+      if (channelUID == 0)
+      {
+        tag.SetClientChannelUid(PVR_TIMER_ANY_CHANNEL);
+      }
+      else if (m_channels.m_channelDetails.find(channelUID) == m_channels.m_channelDetails.end())
+      {
+        kodi::Log(ADDON_LOG_DEBUG, "Invalid channel uid %d", channelUID);
+        tag.SetClientChannelUid(PVR_CHANNEL_INVALID_UID);
+      }
+      else
+      {
+        tag.SetClientChannelUid(channelUID);
+      }
       tag.SetTimerType(pRulesNode->FirstChildElement("EPGTitle") ? TIMER_REPEATING_EPG : TIMER_REPEATING_MANUAL);
 
       std::string buffer;
@@ -93,8 +106,8 @@ PVR_ERROR Timers::GetTimers(kodi::addon::PVRTimersResultSet& results)
 
       if (recordingType == 1 || recordingType == 2)
       {
-        tag.SetStartTime(0);
-        tag.SetEndTime(0);
+        tag.SetStartTime(TIMER_DATE_MIN);
+        tag.SetEndTime(TIMER_DATE_MIN);
         tag.SetStartAnyTime(true);
         tag.SetEndAnyTime(true);
       }
@@ -104,6 +117,10 @@ PVR_ERROR Timers::GetTimers(kodi::addon::PVRTimersResultSet& results)
           tag.SetStartTime(stoll(buffer));
         if (XMLUtils::GetString(pRulesNode, "EndTimeTicks", buffer))
           tag.SetEndTime(stoll(buffer));
+        if (recordingType == 7)
+        {
+          tag.SetEPGSearchString(TYPE_7_TITLE);
+        }
       }
 
       // keyword recordings
@@ -113,8 +130,8 @@ PVR_ERROR Timers::GetTimers(kodi::addon::PVRTimersResultSet& results)
         if (advancedRulesText.find("KEYWORD: ") != std::string::npos)
         {
           tag.SetTimerType(TIMER_REPEATING_KEYWORD);
-          tag.SetStartTime(0);
-          tag.SetEndTime(0);
+          tag.SetStartTime(TIMER_DATE_MIN);
+          tag.SetEndTime(TIMER_DATE_MIN);
           tag.SetStartAnyTime(true);
           tag.SetEndAnyTime(true);
           tag.SetEPGSearchString(advancedRulesText.substr(9));
@@ -122,8 +139,8 @@ PVR_ERROR Timers::GetTimers(kodi::addon::PVRTimersResultSet& results)
         else
         {
           tag.SetTimerType(TIMER_REPEATING_ADVANCED);
-          tag.SetStartTime(0);
-          tag.SetEndTime(0);
+          tag.SetStartTime(TIMER_DATE_MIN);
+          tag.SetEndTime(TIMER_DATE_MIN);
           tag.SetStartAnyTime(true);
           tag.SetEndAnyTime(true);
           tag.SetFullTextEpgSearch(true);
@@ -190,7 +207,7 @@ PVR_ERROR Timers::GetTimers(kodi::addon::PVRTimersResultSet& results)
       XMLUtils::GetString(pRecurringNode, "name", buffer);
       tag.SetTitle(buffer);
       bool state = true;
-      XMLUtils::GetBoolean(pRulesNode, "enabled", state);
+      XMLUtils::GetBoolean(pMatchRulesNode, "enabled", state);
       if (state == false)
           tag.SetState(PVR_TIMER_STATE_DISABLED);
       else
@@ -402,6 +419,7 @@ PVR_ERROR Timers::GetTimerTypes(std::vector<kodi::addon::PVRTimerType>& types)
 
   static const unsigned int TIMER_REPEATING_MANUAL_ATTRIBS
     = PVR_TIMER_TYPE_IS_REPEATING |
+      static_cast<int>(m_settings.m_backendVersion >= ENABLE_DISABLE_VERSION ? PVR_TIMER_TYPE_SUPPORTS_ENABLE_DISABLE : PVR_TIMER_TYPE_ATTRIBUTE_NONE) |
       PVR_TIMER_TYPE_SUPPORTS_START_END_MARGIN |
       PVR_TIMER_TYPE_SUPPORTS_RECORDING_GROUP |
       PVR_TIMER_TYPE_SUPPORTS_WEEKDAYS |
@@ -409,6 +427,7 @@ PVR_ERROR Timers::GetTimerTypes(std::vector<kodi::addon::PVRTimerType>& types)
 
   static const unsigned int TIMER_REPEATING_EPG_ATTRIBS
     = PVR_TIMER_TYPE_IS_REPEATING |
+      static_cast<int>(m_settings.m_backendVersion >= ENABLE_DISABLE_VERSION ? PVR_TIMER_TYPE_SUPPORTS_ENABLE_DISABLE : PVR_TIMER_TYPE_ATTRIBUTE_NONE) |
       PVR_TIMER_TYPE_SUPPORTS_START_END_MARGIN |
       PVR_TIMER_TYPE_SUPPORTS_WEEKDAYS |
       PVR_TIMER_TYPE_SUPPORTS_RECORDING_GROUP |
@@ -428,7 +447,8 @@ PVR_ERROR Timers::GetTimerTypes(std::vector<kodi::addon::PVRTimerType>& types)
       PVR_TIMER_TYPE_SUPPORTS_START_END_MARGIN;
 
   static const unsigned int TIMER_REPEATING_KEYWORD_ATTRIBS
-      = PVR_TIMER_TYPE_IS_REPEATING |
+    = PVR_TIMER_TYPE_IS_REPEATING |
+      static_cast<int>(m_settings.m_backendVersion >= ENABLE_DISABLE_VERSION ? PVR_TIMER_TYPE_SUPPORTS_ENABLE_DISABLE : PVR_TIMER_TYPE_ATTRIBUTE_NONE) |
       PVR_TIMER_TYPE_SUPPORTS_RECORD_ONLY_NEW_EPISODES |
       PVR_TIMER_TYPE_SUPPORTS_ANY_CHANNEL |
       PVR_TIMER_TYPE_SUPPORTS_MAX_RECORDINGS;
@@ -585,13 +605,39 @@ PVR_ERROR Timers::AddTimer(const kodi::addon::PVRTimer& timer)
   else
     strcpy(preventDuplicates, "false");
 
+  std::string enabled;
+  if (m_settings.m_backendVersion >= ENABLE_DISABLE_VERSION)
+  {
+    // NextPVR cannot create new disabled timers
+    if (timer.GetState() == PVR_TIMER_STATE_DISABLED)
+      if (timer.GetClientIndex() != PVR_TIMER_NO_CLIENT_INDEX)
+      {
+        enabled = "&enabled=false";
+      }
+      else
+      {
+        kodi::Log(ADDON_LOG_ERROR, "Cannot create a new disabled timer");
+        return PVR_ERROR_INVALID_PARAMETERS;
+      }
+    else if (timer.GetState() == PVR_TIMER_STATE_SCHEDULED)
+    {
+      enabled = "&enabled=true";
+    }
+  }
+
   const std::string encodedName = UriEncode(timer.GetTitle());
   const std::string encodedKeyword = UriEncode(timer.GetEPGSearchString());
   const std::string days = GetDayString(timer.GetWeekdays());
   const std::string directory = UriEncode(m_settings.m_recordingDirectories[timer.GetRecordingGroup()]);
-  const std::string oidKey = std::to_string(timer.GetEPGUid()) + ":" + std::to_string(timer.GetClientChannelUid());
-  const int epgOid = m_epgOidLookup[oidKey];
-  kodi::Log(ADDON_LOG_DEBUG, "TIMER_%d %s", epgOid, oidKey.c_str());
+
+  int epgOid = 0;
+  if (timer.GetEPGUid() > 0)
+  {
+    const std::string oidKey = std::to_string(timer.GetEPGUid()) + ":" + std::to_string(timer.GetClientChannelUid());
+    epgOid = GetEPGOidForTimer(timer);
+    kodi::Log(ADDON_LOG_DEBUG, "TIMER %d %s", epgOid, oidKey.c_str());
+  }
+
   std::string request;
 
   int marginStart = timer.GetMarginStart();
@@ -607,11 +653,12 @@ PVR_ERROR Timers::AddTimer(const kodi::addon::PVRTimer& timer)
   case TIMER_ONCE_MANUAL:
     kodi::Log(ADDON_LOG_DEBUG, "TIMER_ONCE_MANUAL");
     // build one-off recording request
-    request = kodi::tools::StringUtils::Format("recording.save&name=%s&channel=%d&time_t=%d&duration=%d&pre_padding=%d&post_padding=%d&directory_id=%s",
+    request = kodi::tools::StringUtils::Format("recording.save&name=%s&recording_id=%d&channel=%d&time_t=%d&duration=%d&pre_padding=%d&post_padding=%d&directory_id=%s",
       encodedName.c_str(),
+      timer.GetClientIndex(),
       timer.GetClientChannelUid(),
-      (int)timer.GetStartTime(),
-      (int)(timer.GetEndTime() - timer.GetStartTime()),
+      static_cast<int>(timer.GetStartTime()),
+      static_cast<int>(timer.GetEndTime() - timer.GetStartTime()),
       marginStart,
       marginEnd,
       directory.c_str()
@@ -630,36 +677,57 @@ PVR_ERROR Timers::AddTimer(const kodi::addon::PVRTimer& timer)
 
   case TIMER_REPEATING_EPG:
     if (timer.GetClientChannelUid() == PVR_TIMER_ANY_CHANNEL)
+    // Fake a manual recording not a specific type in NextPVR
     {
-      // Fake a manual recording
-      kodi::Log(ADDON_LOG_DEBUG, "TIMER_REPEATING_EPG ANY CHANNEL");
-      std::string title = encodedName + "%";
-      request = kodi::tools::StringUtils::Format("recording.recurring.save&name=%s&channel_id=%d&start_time=%d&end_time=%d&keep=%d&pre_padding=%d&post_padding=%d&day_mask=%s&directory_id=%s&keyword=%s",
-        encodedName.c_str(),
-        timer.GetClientChannelUid(),
-        (int)timer.GetStartTime(),
-        (int)timer.GetEndTime(),
-        timer.GetMaxRecordings(),
-        marginStart,
-        marginEnd,
-        days.c_str(),
-        directory.c_str(),
-        title.c_str()
-        );
+      if (timer.GetEPGSearchString() == TYPE_7_TITLE)
+      {
+        kodi::Log(ADDON_LOG_DEBUG, "TIMER_REPEATING_EPG ANY CHANNEL - TYPE 7");
+        request = kodi::tools::StringUtils::Format("recording.recurring.save&type=7&recurring_id=%d&start_time=%d&end_time=%d&keep=%d&pre_padding=%d&post_padding=%d&day_mask=%s&directory_id=%s%s",
+          timer.GetClientIndex(),
+          static_cast<int>(timer.GetStartTime()),
+          static_cast<int>(timer.GetEndTime()),
+          timer.GetMaxRecordings(),
+          marginStart,
+          marginEnd,
+          days.c_str(),
+          directory.c_str(),
+          enabled.c_str()
+          );
+      }
+      else
+      {
+        kodi::Log(ADDON_LOG_DEBUG, "TIMER_REPEATING_EPG ANY CHANNEL");
+        std::string title = encodedName + "%";
+        request = kodi::tools::StringUtils::Format("recording.recurring.save&name=%s&channel_id=%d&start_time=%d&end_time=%d&keep=%d&pre_padding=%d&post_padding=%d&day_mask=%s&directory_id=%s&keyword=%s%s",
+          encodedName.c_str(),
+          0,
+          static_cast<int>(timer.GetStartTime()),
+          static_cast<int>(timer.GetEndTime()),
+          timer.GetMaxRecordings(),
+          marginStart,
+          marginEnd,
+          days.c_str(),
+          directory.c_str(),
+          title.c_str(),
+          enabled.c_str()
+          );
+      }
     }
     else
     {
       kodi::Log(ADDON_LOG_DEBUG, "TIMER_REPEATING_EPG");
       // build recurring recording request
-      request = kodi::tools::StringUtils::Format("recording.recurring.save&recurring_id=%d&event_id=%d&keep=%d&pre_padding=%d&post_padding=%d&day_mask=%s&directory_id=%s&only_new=%s",
+      request = kodi::tools::StringUtils::Format("recording.recurring.save&recurring_id=%d&channel_id=%d&event_id=%d&keep=%d&pre_padding=%d&post_padding=%d&day_mask=%s&directory_id=%s&only_new=%s%s",
         timer.GetClientIndex(),
+        timer.GetClientChannelUid(),
         epgOid,
         timer.GetMaxRecordings(),
         marginStart,
         marginEnd,
         days.c_str(),
         directory.c_str(),
-        preventDuplicates
+        preventDuplicates,
+        enabled.c_str()
         );
     }
     break;
@@ -667,53 +735,55 @@ PVR_ERROR Timers::AddTimer(const kodi::addon::PVRTimer& timer)
   case TIMER_REPEATING_MANUAL:
     kodi::Log(ADDON_LOG_DEBUG, "TIMER_REPEATING_MANUAL");
     // build manual recurring request
-    request = kodi::tools::StringUtils::Format("recording.recurring.save&recurring_id=%d&name=%s&channel_id=%d&start_time=%d&end_time=%d&keep=%d&pre_padding=%d&post_padding=%d&day_mask=%s&directory_id=%s",
+    request = kodi::tools::StringUtils::Format("recording.recurring.save&recurring_id=%d&name=%s&channel_id=%d&start_time=%d&end_time=%d&keep=%d&pre_padding=%d&post_padding=%d&day_mask=%s&directory_id=%s%s",
       timer.GetClientIndex(),
       encodedName.c_str(),
       timer.GetClientChannelUid(),
-      (int)timer.GetStartTime(),
-      (int)timer.GetEndTime(),
+      static_cast<int>(timer.GetStartTime()),
+      static_cast<int>(timer.GetEndTime()),
       timer.GetMaxRecordings(),
       marginStart,
       marginEnd,
       days.c_str(),
-      directory.c_str()
+      directory.c_str(),
+      enabled.c_str()
       );
     break;
 
   case TIMER_REPEATING_KEYWORD:
     kodi::Log(ADDON_LOG_DEBUG, "TIMER_REPEATING_KEYWORD");
     // build manual recurring request
-    request = kodi::tools::StringUtils::Format("recording.recurring.save&recurring_id=%d&name=%s&channel_id=%d&start_time=%d&end_time=%d&keep=%d&pre_padding=%d&post_padding=%d&directory_id=%s&keyword=%s&only_new=%s",
+    request = kodi::tools::StringUtils::Format("recording.recurring.save&recurring_id=%d&name=%s&channel_id=%d&start_time=%d&end_time=%d&keep=%d&pre_padding=%d&post_padding=%d&directory_id=%s&keyword=%s&only_new=%s%s",
       timer.GetClientIndex(),
       encodedName.c_str(),
       timer.GetClientChannelUid(),
-      (int)timer.GetStartTime(),
-      (int)timer.GetEndTime(),
+      static_cast<int>(timer.GetStartTime()),
+      static_cast<int>(timer.GetEndTime()),
       timer.GetMaxRecordings(),
       marginStart,
       marginEnd,
       directory.c_str(),
       encodedKeyword.c_str(),
-      preventDuplicates
+      preventDuplicates,
+      enabled.c_str()
       );
-    break;
 
   case TIMER_REPEATING_ADVANCED:
     kodi::Log(ADDON_LOG_DEBUG, "TIMER_REPEATING_ADVANCED");
     // build manual advanced recurring request
-    request = kodi::tools::StringUtils::Format("recording.recurring.save&recurring_type=advanced&recurring_id=%d&name=%s&channel_id=%d&start_time=%d&end_time=%d&keep=%d&pre_padding=%d&post_padding=%d&directory_id=%s&advanced=%s&only_new=%s",
+    request = kodi::tools::StringUtils::Format("recording.recurring.save&recurring_type=advanced&recurring_id=%d&name=%s&channel_id=%d&start_time=%d&end_time=%d&keep=%d&pre_padding=%d&post_padding=%d&directory_id=%s&advanced=%s&only_new=%s%s",
       timer.GetClientIndex(),
       encodedName.c_str(),
       timer.GetClientChannelUid(),
-      (int)timer.GetStartTime(),
-      (int)timer.GetEndTime(),
+      static_cast<int>(timer.GetStartTime()),
+      static_cast<int>(timer.GetEndTime()),
       timer.GetMaxRecordings(),
       marginStart,
       marginEnd,
       directory.c_str(),
       encodedKeyword.c_str(),
-      preventDuplicates
+      preventDuplicates,
+      enabled.c_str()
       );
     break;
   }
@@ -758,3 +828,28 @@ PVR_ERROR Timers::UpdateTimer(const kodi::addon::PVRTimer& timer)
 {
   return AddTimer(timer);
 }
+
+int Timers::GetEPGOidForTimer(const kodi::addon::PVRTimer& timer)
+{
+  std::string request = kodi::tools::StringUtils::Format("channel.listings&channel_id=%d&start=%d&end=%d",
+    timer.GetClientChannelUid(),timer.GetEPGUid() - 1, timer.GetEPGUid());
+
+  tinyxml2::XMLDocument doc;
+  int epgOid = 0;
+  if (m_request.DoMethodRequest(request, doc) == tinyxml2::XML_SUCCESS)
+  {
+    tinyxml2::XMLNode* listingsNode = doc.RootElement()->FirstChildElement("listings");
+    for (tinyxml2::XMLNode* pListingNode = listingsNode->FirstChildElement("l"); pListingNode; pListingNode = pListingNode->NextSiblingElement())
+    {
+      std::string endTime;
+      XMLUtils::GetString(pListingNode, "end", endTime);
+      endTime.resize(10);
+      if (atoi(endTime.c_str()) == timer.GetEPGUid())
+      {
+        epgOid = XMLUtils::GetIntValue(pListingNode, "id");
+        break;
+      }
+    }
+  }
+  return epgOid;
+}
